@@ -1,10 +1,14 @@
 use super::range_wrapper::RangeInclusiveStartWrapper;
+use crate::range_wrapper::RangeInclusiveEndWrapper;
 use crate::std_ext::*;
 use alloc::collections::BTreeMap;
+use core::borrow::Borrow;
+use core::cmp::Ordering;
 use core::fmt::{self, Debug};
-use core::iter::FromIterator;
+use core::hash::Hash;
+use core::iter::{DoubleEndedIterator, FromIterator};
 use core::marker::PhantomData;
-use core::ops::RangeInclusive;
+use core::ops::{RangeFrom, RangeInclusive};
 use core::prelude::v1::*;
 
 #[cfg(feature = "serde1")]
@@ -25,10 +29,10 @@ use serde::{
 /// because adjacent ranges can be detected using equality of range ends alone.)
 ///
 /// You can provide these functions either by implementing the
-/// [`StepLite`](crate::StepLite) trait for your key type `K`, or,
+/// [`StepLite`] trait for your key type `K`, or,
 /// if this is impossible because of Rust's "orphan rules",
 /// you can provide equivalent free functions using the `StepFnsT` type parameter.
-/// [`StepLite`](crate::StepLite) is implemented for all standard integer types,
+/// [`StepLite`] is implemented for all standard integer types,
 /// but not for any third party crate types.
 #[derive(Clone)]
 pub struct RangeInclusiveMap<K, V, StepFnsT = K> {
@@ -38,22 +42,107 @@ pub struct RangeInclusiveMap<K, V, StepFnsT = K> {
     _phantom: PhantomData<StepFnsT>,
 }
 
-impl<K, V> Default for RangeInclusiveMap<K, V, K>
-where
-    K: Ord + Clone + StepLite,
-    V: Eq + Clone,
-{
+impl<K, V, StepFnsT> Default for RangeInclusiveMap<K, V, StepFnsT> {
     fn default() -> Self {
-        Self::new()
+        Self {
+            btm: BTreeMap::default(),
+            _phantom: PhantomData,
+        }
+    }
+}
+
+impl<K, V, StepFnsT> Hash for RangeInclusiveMap<K, V, StepFnsT>
+where
+    K: Hash,
+    V: Hash,
+{
+    fn hash<H: core::hash::Hasher>(&self, state: &mut H) {
+        state.write_usize(self.btm.len());
+        for elt in self.iter() {
+            elt.hash(state);
+        }
+    }
+}
+
+impl<K, V, StepFnsT> PartialEq for RangeInclusiveMap<K, V, StepFnsT>
+where
+    K: PartialEq,
+    V: PartialEq,
+{
+    fn eq(&self, other: &RangeInclusiveMap<K, V, StepFnsT>) -> bool {
+        self.btm == other.btm
+    }
+}
+
+impl<K, V, StepFnsT> Eq for RangeInclusiveMap<K, V, StepFnsT>
+where
+    K: Eq,
+    V: Eq,
+{
+}
+
+impl<K, V, StepFnsT> PartialOrd for RangeInclusiveMap<K, V, StepFnsT>
+where
+    K: PartialOrd,
+    V: PartialOrd,
+{
+    #[inline]
+    fn partial_cmp(&self, other: &RangeInclusiveMap<K, V, StepFnsT>) -> Option<Ordering> {
+        self.btm.partial_cmp(&other.btm)
+    }
+}
+
+impl<K, V, StepFnsT> Ord for RangeInclusiveMap<K, V, StepFnsT>
+where
+    K: Ord,
+    V: Ord,
+{
+    #[inline]
+    fn cmp(&self, other: &RangeInclusiveMap<K, V, StepFnsT>) -> Ordering {
+        self.btm.cmp(&other.btm)
+    }
+}
+
+#[cfg(feature = "quickcheck")]
+impl<K, V> quickcheck::Arbitrary for RangeInclusiveMap<K, V>
+where
+    K: quickcheck::Arbitrary + Ord + StepLite,
+    V: quickcheck::Arbitrary + PartialEq,
+{
+    fn arbitrary(g: &mut quickcheck::Gen) -> Self {
+        // REVISIT: allocation could be avoided if Gen::gen_size were public (https://github.com/BurntSushi/quickcheck/issues/326#issue-2653601170)
+        <alloc::vec::Vec<(RangeInclusive<_>, _)>>::arbitrary(g)
+            .into_iter()
+            .filter(|(range, _)| !range.is_empty())
+            .collect()
+    }
+}
+
+impl<K, V, StepFnsT> RangeInclusiveMap<K, V, StepFnsT> {
+    /// Gets an iterator over all pairs of key range and value,
+    /// ordered by key range.
+    ///
+    /// The iterator element type is `(&'a RangeInclusive<K>, &'a V)`.
+    pub fn iter(&self) -> Iter<'_, K, V> {
+        Iter {
+            inner: self.btm.iter(),
+        }
     }
 }
 
 impl<K, V> RangeInclusiveMap<K, V, K>
 where
     K: Ord + Clone + StepLite,
-    V: Eq + Clone,
+    V: PartialEq + Clone,
 {
     /// Makes a new empty `RangeInclusiveMap`.
+    #[cfg(feature = "const_fn")]
+    pub const fn new() -> Self {
+        Self::new_with_step_fns()
+    }
+
+    /// Makes a new empty `RangeInclusiveMap`.
+    #[cfg(not(feature = "const_fn"))]
     pub fn new() -> Self {
         Self::new_with_step_fns()
     }
@@ -62,7 +151,7 @@ where
 impl<K, V, StepFnsT> RangeInclusiveMap<K, V, StepFnsT>
 where
     K: Ord + Clone,
-    V: Eq + Clone,
+    V: PartialEq + Clone,
     StepFnsT: StepFns<K>,
 {
     /// Makes a new empty `RangeInclusiveMap`, specifying successor and
@@ -79,6 +168,7 @@ where
     ///
     /// See [this issue](https://github.com/rust-lang/rust/issues/42168)
     /// for details about that stabilization process.
+    #[cfg(not(feature = "const_fn"))]
     pub fn new_with_step_fns() -> Self {
         Self {
             btm: BTreeMap::new(),
@@ -86,6 +176,13 @@ where
         }
     }
 
+    #[cfg(feature = "const_fn")]
+    pub const fn new_with_step_fns() -> Self {
+        Self {
+            btm: BTreeMap::new(),
+            _phantom: PhantomData,
+        }
+    }
     /// Returns a reference to the value corresponding to the given key,
     /// if the key is covered by any range in the map.
     pub fn get(&self, key: &K) -> Option<&V> {
@@ -106,7 +203,7 @@ where
             .filter(|(range_start_wrapper, _value)| {
                 // Does the only candidate range contain
                 // the requested key?
-                range_start_wrapper.range.contains(key)
+                range_start_wrapper.contains(key)
             })
             .map(|(range_start_wrapper, value)| (&range_start_wrapper.range, value))
     }
@@ -116,14 +213,19 @@ where
         self.get(key).is_some()
     }
 
-    /// Gets an iterator over all pairs of key range and value,
-    /// ordered by key range.
-    ///
-    /// The iterator element type is `(&'a RangeInclusive<K>, &'a V)`.
-    pub fn iter(&self) -> Iter<'_, K, V> {
-        Iter {
-            inner: self.btm.iter(),
-        }
+    /// Clears the map, removing all elements.
+    pub fn clear(&mut self) {
+        self.btm.clear();
+    }
+
+    /// Returns the number of elements in the map.
+    pub fn len(&self) -> usize {
+        self.btm.len()
+    }
+
+    /// Returns true if the map contains no elements.
+    pub fn is_empty(&self) -> bool {
+        self.btm.is_empty()
     }
 
     /// Insert a pair of key range and value into the map.
@@ -167,7 +269,10 @@ where
         // if both of the above cases exist.
         let mut candidates = self
             .btm
-            .range((Bound::Unbounded, Bound::Included(&new_range_start_wrapper)))
+            .range::<RangeInclusiveStartWrapper<K>, (
+                Bound<&RangeInclusiveStartWrapper<K>>,
+                Bound<&RangeInclusiveStartWrapper<K>>,
+            )>((Bound::Unbounded, Bound::Included(&new_range_start_wrapper)))
             .rev()
             .take(2)
             .filter(|(stored_range_start_wrapper, _stored_value)| {
@@ -176,8 +281,7 @@ where
                 // (Remember that it might actually cover the _whole_
                 // range to insert and then some.)
                 stored_range_start_wrapper
-                    .range
-                    .touches::<StepFnsT>(&new_range_start_wrapper.range)
+                    .touches::<StepFnsT>(&new_range_start_wrapper.end_wrapper.range)
             });
         if let Some(mut candidate) = candidates.next() {
             // Or the one before it if both cases described above exist.
@@ -189,7 +293,7 @@ where
             self.adjust_touching_ranges_for_insert(
                 stored_range_start_wrapper,
                 stored_value,
-                &mut new_range_start_wrapper.range,
+                &mut new_range_start_wrapper.end_wrapper.range,
                 &new_value,
             );
         }
@@ -205,13 +309,16 @@ where
         //
         // REVISIT: Possible micro-optimisation: `impl Borrow<T> for RangeInclusiveStartWrapper<T>`
         // and use that to search here, to avoid constructing another `RangeInclusiveStartWrapper`.
-        let second_last_possible_start = new_range_start_wrapper.range.end().clone();
+        let second_last_possible_start = new_range_start_wrapper.end().clone();
         let second_last_possible_start = RangeInclusiveStartWrapper::new(
             second_last_possible_start.clone()..=second_last_possible_start,
         );
         while let Some((stored_range_start_wrapper, stored_value)) = self
             .btm
-            .range((
+            .range::<RangeInclusiveStartWrapper<K>, (
+                Bound<&RangeInclusiveStartWrapper<K>>,
+                Bound<&RangeInclusiveStartWrapper<K>>,
+            )>((
                 Bound::Included(&new_range_start_wrapper),
                 // We would use something like `Bound::Included(&last_possible_start)`,
                 // but making `last_possible_start` might cause arithmetic overflow;
@@ -224,10 +331,9 @@ where
             // end of the subset of stored ranges we want to consider,
             // in part because we use `Bound::Unbounded` above.
             // (See comments up there, and in the individual cases below.)
-            let stored_start = stored_range_start_wrapper.range.start();
-            if *stored_start > *second_last_possible_start.range.start() {
-                let latest_possible_start =
-                    StepFnsT::add_one(second_last_possible_start.range.start());
+            let stored_start = stored_range_start_wrapper.start();
+            if *stored_start > *second_last_possible_start.start() {
+                let latest_possible_start = StepFnsT::add_one(second_last_possible_start.start());
                 if *stored_start > latest_possible_start {
                     // We're beyond the last stored range that could be relevant.
                     // Avoid wasting time on irrelevant ranges, or even worse, looping forever.
@@ -253,7 +359,7 @@ where
             self.adjust_touching_ranges_for_insert(
                 stored_range_start_wrapper,
                 stored_value,
-                &mut new_range_start_wrapper.range,
+                &mut new_range_start_wrapper.end_wrapper.range,
                 &new_value,
             );
         }
@@ -295,12 +401,15 @@ where
         // whose start is less than or equal to the start of the range to insert.
         if let Some((stored_range_start_wrapper, stored_value)) = self
             .btm
-            .range((Bound::Unbounded, Bound::Included(&range_start_wrapper)))
+            .range::<RangeInclusiveStartWrapper<K>, (
+                Bound<&RangeInclusiveStartWrapper<K>>,
+                Bound<&RangeInclusiveStartWrapper<K>>,
+            )>((Bound::Unbounded, Bound::Included(&range_start_wrapper)))
             .next_back()
             .filter(|(stored_range_start_wrapper, _stored_value)| {
                 // Does the only candidate range overlap
                 // the range to insert?
-                stored_range_start_wrapper.range.overlaps(range)
+                stored_range_start_wrapper.overlaps(range)
             })
             .map(|(stored_range_start_wrapper, stored_value)| {
                 (stored_range_start_wrapper.clone(), stored_value.clone())
@@ -325,7 +434,10 @@ where
             RangeInclusiveStartWrapper::new(range.end().clone()..=range.end().clone());
         while let Some((stored_range_start_wrapper, stored_value)) = self
             .btm
-            .range((
+            .range::<RangeInclusiveStartWrapper<K>, (
+                Bound<&RangeInclusiveStartWrapper<K>>,
+                Bound<&RangeInclusiveStartWrapper<K>>,
+            )>((
                 Bound::Excluded(&range_start_wrapper),
                 Bound::Included(&new_range_end_as_start),
             ))
@@ -358,9 +470,8 @@ where
             // This means that no matter how big or where the stored range is,
             // we will expand the new range's bounds to subsume it,
             // and then delete the stored range.
-            let new_start =
-                min(new_range.start(), stored_range_start_wrapper.range.start()).clone();
-            let new_end = max(new_range.end(), stored_range_start_wrapper.range.end()).clone();
+            let new_start = min(new_range.start(), stored_range_start_wrapper.start()).clone();
+            let new_end = max(new_range.end(), stored_range_start_wrapper.end()).clone();
             *new_range = new_start..=new_end;
             self.btm.remove(&stored_range_start_wrapper);
         } else {
@@ -370,22 +481,22 @@ where
                 // Delete the stored range, and then add back between
                 // 0 and 2 subranges at the ends of the range to insert.
                 self.btm.remove(&stored_range_start_wrapper);
-                if stored_range_start_wrapper.range.start() < new_range.start() {
+                if stored_range_start_wrapper.start() < new_range.start() {
                     // Insert the piece left of the range to insert.
                     self.btm.insert(
                         RangeInclusiveStartWrapper::new(
-                            stored_range_start_wrapper.range.start().clone()
+                            stored_range_start_wrapper.start().clone()
                                 ..=StepFnsT::sub_one(new_range.start()),
                         ),
                         stored_value.clone(),
                     );
                 }
-                if stored_range_start_wrapper.range.end() > new_range.end() {
+                if stored_range_start_wrapper.end() > new_range.end() {
                     // Insert the piece right of the range to insert.
                     self.btm.insert(
                         RangeInclusiveStartWrapper::new(
                             StepFnsT::add_one(new_range.end())
-                                ..=stored_range_start_wrapper.range.end().clone(),
+                                ..=stored_range_start_wrapper.end().clone(),
                         ),
                         stored_value,
                     );
@@ -406,7 +517,7 @@ where
         // Delete the stored range, and then add back between
         // 0 and 2 subranges at the ends of the range to insert.
         self.btm.remove(&stored_range_start_wrapper);
-        let stored_range = stored_range_start_wrapper.range;
+        let stored_range = stored_range_start_wrapper.end_wrapper.range;
         if stored_range.start() < range_to_remove.start() {
             // Insert the piece left of the range to insert.
             self.btm.insert(
@@ -433,18 +544,62 @@ where
     ///
     /// The iterator element type is `RangeInclusive<K>`.
     pub fn gaps<'a>(&'a self, outer_range: &'a RangeInclusive<K>) -> Gaps<'a, K, V, StepFnsT> {
-        let keys = self.btm.keys().peekable();
+        let overlap_iter = self.overlapping(outer_range);
         Gaps {
-            done: false,
-            outer_range,
-            keys,
+            candidate_needs_plus_one: false,
+            candidate_start: outer_range.start(),
+            query_end: outer_range.end(),
+            btm_range_iter: overlap_iter.btm_range_iter,
             // We'll start the candidate range at the start of the outer range
             // without checking what's there. Each time we yield an item,
             // we'll skip any ranges we find before the next gap.
-            candidate_start: outer_range.start().clone(),
             _phantom: PhantomData,
         }
     }
+
+    /// Gets an iterator over all the stored ranges that are
+    /// either partially or completely overlapped by the given range.
+    pub fn overlapping<R: Borrow<RangeInclusive<K>>>(
+        &'_ self,
+        range: R,
+    ) -> Overlapping<'_, K, V, R> {
+        // Find the first matching stored range by its _end_,
+        // using sneaky layering and `Borrow` implementation. (See `range_wrappers` module.)
+        let start_sliver = RangeInclusiveEndWrapper::new(
+            range.borrow().start().clone()..=range.borrow().start().clone(),
+        );
+        let btm_range_iter = self
+            .btm
+            .range::<RangeInclusiveEndWrapper<K>, RangeFrom<&RangeInclusiveEndWrapper<K>>>(
+                &start_sliver..,
+            );
+        Overlapping {
+            query_range: range,
+            btm_range_iter,
+        }
+    }
+
+    /// Returns `true` if any range in the map completely or partially
+    /// overlaps the given range.
+    pub fn overlaps(&self, range: &RangeInclusive<K>) -> bool {
+        self.overlapping(range).next().is_some()
+    }
+
+    /// Returns the first range-value pair in this map, if one exists. The range in this pair is the
+    /// minimum range in the map.
+    pub fn first_range_value(&self) -> Option<(&RangeInclusive<K>, &V)> {
+        self.btm
+            .first_key_value()
+            .map(|(range, value)| (&range.end_wrapper.range, value))
+    }
+
+    /// Returns the last range-value pair in this map, if one exists. The range in this pair is the
+    /// maximum range in the map.
+    pub fn last_range_value(&self) -> Option<(&RangeInclusive<K>, &V)> {
+        self.btm
+            .last_key_value()
+            .map(|(range, value)| (&range.end_wrapper.range, value))
+    }
 }
 
 /// An iterator over the entries of a `RangeInclusiveMap`, ordered by key range.
@@ -466,7 +621,7 @@ where
 {
     type Item = (&'a RangeInclusive<K>, &'a V);
 
-    fn next(&mut self) -> Option<(&'a RangeInclusive<K>, &'a V)> {
+    fn next(&mut self) -> Option<Self::Item> {
         self.inner.next().map(|(by_start, v)| (&by_start.range, v))
     }
 
@@ -475,6 +630,18 @@ where
     }
 }
 
+impl<'a, K, V> DoubleEndedIterator for Iter<'a, K, V>
+where
+    K: 'a,
+    V: 'a,
+{
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.inner
+            .next_back()
+            .map(|(range, value)| (&range.end_wrapper.range, value))
+    }
+}
+
 /// An owning iterator over the entries of a `RangeInclusiveMap`, ordered by key range.
 ///
 /// The iterator element type is `(RangeInclusive<K>, V)`.
@@ -500,20 +667,30 @@ impl<K, V> IntoIterator for RangeInclusiveMap<K, V> {
 impl<K, V> Iterator for IntoIter<K, V> {
     type Item = (RangeInclusive<K>, V);
     fn next(&mut self) -> Option<(RangeInclusive<K>, V)> {
-        self.inner.next().map(|(by_start, v)| (by_start.range, v))
+        self.inner
+            .next()
+            .map(|(by_start, v)| (by_start.end_wrapper.range, v))
     }
     fn size_hint(&self) -> (usize, Option<usize>) {
         self.inner.size_hint()
     }
 }
 
+impl<K, V> DoubleEndedIterator for IntoIter<K, V> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.inner
+            .next_back()
+            .map(|(range, value)| (range.end_wrapper.range, value))
+    }
+}
+
 // We can't just derive this automatically, because that would
 // expose irrelevant (and private) implementation details.
 // Instead implement it in the same way that the underlying BTreeMap does.
 impl<K: Debug, V: Debug> Debug for RangeInclusiveMap<K, V>
 where
     K: Ord + Clone + StepLite,
-    V: Eq + Clone,
+    V: PartialEq + Clone,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_map().entries(self.iter()).finish()
@@ -523,7 +700,7 @@ where
 impl<K, V> FromIterator<(RangeInclusive<K>, V)> for RangeInclusiveMap<K, V>
 where
     K: Ord + Clone + StepLite,
-    V: Eq + Clone,
+    V: PartialEq + Clone,
 {
     fn from_iter<T: IntoIterator<Item = (RangeInclusive<K>, V)>>(iter: T) -> Self {
         let mut range_map = RangeInclusiveMap::new();
@@ -535,7 +712,7 @@ where
 impl<K, V> Extend<(RangeInclusive<K>, V)> for RangeInclusiveMap<K, V>
 where
     K: Ord + Clone + StepLite,
-    V: Eq + Clone,
+    V: PartialEq + Clone,
 {
     fn extend<T: IntoIterator<Item = (RangeInclusive<K>, V)>>(&mut self, iter: T) {
         iter.into_iter().for_each(move |(k, v)| {
@@ -548,7 +725,7 @@ where
 impl<K, V> Serialize for RangeInclusiveMap<K, V>
 where
     K: Ord + Clone + StepLite + Serialize,
-    V: Eq + Clone + Serialize,
+    V: PartialEq + Clone + Serialize,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -567,7 +744,7 @@ where
 impl<'de, K, V> Deserialize<'de> for RangeInclusiveMap<K, V>
 where
     K: Ord + Clone + StepLite + Deserialize<'de>,
-    V: Eq + Clone + Deserialize<'de>,
+    V: PartialEq + Clone + Deserialize<'de>,
 {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
     where
@@ -595,7 +772,7 @@ impl<K, V> RangeInclusiveMapVisitor<K, V> {
 impl<'de, K, V> Visitor<'de> for RangeInclusiveMapVisitor<K, V>
 where
     K: Ord + Clone + StepLite + Deserialize<'de>,
-    V: Eq + Clone + Deserialize<'de>,
+    V: PartialEq + Clone + Deserialize<'de>,
 {
     type Value = RangeInclusiveMap<K, V>;
 
@@ -628,12 +805,10 @@ pub struct Gaps<'a, K, V, StepFnsT> {
     /// avoid overflowing when dealing with inclusive ranges.
     ///
     /// All other things here are ignored if `done` is `true`.
-    done: bool,
-    outer_range: &'a RangeInclusive<K>,
-    keys: core::iter::Peekable<
-        alloc::collections::btree_map::Keys<'a, RangeInclusiveStartWrapper<K>, V>,
-    >,
-    candidate_start: K,
+    candidate_needs_plus_one: bool,
+    candidate_start: &'a K,
+    query_end: &'a K,
+    btm_range_iter: alloc::collections::btree_map::Range<'a, RangeInclusiveStartWrapper<K>, V>,
     _phantom: PhantomData<StepFnsT>,
 }
 
@@ -653,71 +828,307 @@ where
     type Item = RangeInclusive<K>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        if self.done || self.candidate_start > *self.outer_range.end() {
-            // We've already passed the end of the outer range;
-            // there are no more gaps to find.
-            return None;
-        }
+        for overlap in self.btm_range_iter.by_ref() {
+            let overlap = overlap.0;
 
-        // Skip past any ranges to find the first value that might
-        // be the start of a gap.
-        while let Some(item) = self.keys.peek() {
-            if *item.range.start() <= self.candidate_start {
-                // Next potential gap starts after this range, and at least as
-                // late as the start of the outer range.
-                self.candidate_start = if item.range.end() < self.outer_range.start() {
-                    self.outer_range.start().clone()
-                } else if item.range.end() >= self.outer_range.end() {
-                    // We're done; there no more room for gaps.
-                    self.done = true;
-                    return None;
-                } else {
-                    StepFnsT::add_one(item.range.end())
-                };
-                let _ = self.keys.next();
+            // If the range in the map has advanced beyond the query range, return
+            // any tail gap.
+            if *self.query_end < *overlap.start() {
+                break;
+            }
+
+            let candidate_needs_plus_one =
+                core::mem::replace(&mut self.candidate_needs_plus_one, true);
+
+            let cur_candidate_start = core::mem::replace(&mut self.candidate_start, overlap.end());
+
+            let cur_candidate_start = if candidate_needs_plus_one {
+                StepFnsT::add_one(cur_candidate_start)
             } else {
-                // There's a gap before this item!
-                let end = if item.range.start() <= self.outer_range.end() {
-                    // It starts within the outer range; return the gap directly.
-                    StepFnsT::sub_one(item.range.start())
-                } else {
-                    // Next item starts beyond the end of the outer range;
-                    // end our gap at the end of the outer range.
-                    self.outer_range.end().clone()
-                };
-                let gap = self.candidate_start.clone()..=end;
-                // Next potential gap starts after this range.
-                if gap.end() < self.outer_range.end() {
-                    // We can safely add one; there's at least one more element in the domain.
-                    self.candidate_start = StepFnsT::add_one(gap.end());
-                } else {
-                    // The gap went right up to the end of the outer range;
-                    // there can be no more gaps.
-                    self.done = true;
-                }
+                cur_candidate_start.clone()
+            };
+
+            if cur_candidate_start < *overlap.start() {
+                let gap = cur_candidate_start..=StepFnsT::sub_one(overlap.start());
                 return Some(gap);
             }
         }
 
-        // If we got this far, the only other gap that might be
-        // is at the end of the outer range.
-        if self.candidate_start <= *self.outer_range.end() {
+        // Now that we've run out of items, the only other possible
+        // gap is one at the end of the outer range.
+        let candidate_needs_plus_one = core::mem::replace(&mut self.candidate_needs_plus_one, true);
+
+        let cur_candidate_start = core::mem::replace(&mut self.candidate_start, self.query_end);
+        if candidate_needs_plus_one {
+            if *cur_candidate_start < *self.query_end {
+                return Some(StepFnsT::add_one(cur_candidate_start)..=self.query_end.clone());
+            }
+        } else if *cur_candidate_start <= *self.query_end {
             // There's a gap at the end!
-            let gap = self.candidate_start.clone()..=self.outer_range.end().clone();
-            // We're done; don't try to find any more gaps.
-            self.done = true;
-            Some(gap)
+            return Some(cur_candidate_start.clone()..=self.query_end.clone());
+        }
+
+        None
+    }
+}
+
+/// An iterator over all stored ranges partially or completely
+/// overlapped by a given range.
+///
+/// The iterator element type is `(&'a RangeInclusive<K>, &'a V)`.
+///
+/// This `struct` is created by the [`overlapping`] method on [`RangeInclusiveMap`]. See its
+/// documentation for more.
+///
+/// [`overlapping`]: RangeInclusiveMap::overlapping
+pub struct Overlapping<'a, K, V, R: Borrow<RangeInclusive<K>> = &'a RangeInclusive<K>> {
+    query_range: R,
+    btm_range_iter: alloc::collections::btree_map::Range<'a, RangeInclusiveStartWrapper<K>, V>,
+}
+
+// `Overlapping` is always fused. (See definition of `next` below.)
+impl<'a, K, V, R: Borrow<RangeInclusive<K>>> core::iter::FusedIterator for Overlapping<'a, K, V, R> where
+    K: Ord + Clone
+{
+}
+
+impl<'a, K, V, R: Borrow<RangeInclusive<K>>> Iterator for Overlapping<'a, K, V, R>
+where
+    K: Ord + Clone,
+{
+    type Item = (&'a RangeInclusive<K>, &'a V);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if let Some((k, v)) = self.btm_range_iter.next() {
+            if k.start() <= self.query_range.borrow().end() {
+                Some((&k.range, v))
+            } else {
+                // The rest of the items in the underlying iterator
+                // are past the query range. We can keep taking items
+                // from that iterator and this will remain true,
+                // so this is enough to make the iterator fused.
+                None
+            }
         } else {
-            // We got to the end; there can't be any more gaps.
             None
         }
     }
 }
 
+impl<'a, K, V, R: Borrow<RangeInclusive<K>>> DoubleEndedIterator for Overlapping<'a, K, V, R>
+where
+    K: Ord + Clone,
+{
+    fn next_back(&mut self) -> Option<Self::Item> {
+        while let Some((k, v)) = self.btm_range_iter.next_back() {
+            if k.start() <= self.query_range.borrow().end() {
+                return Some((&k.range, v));
+            }
+        }
+
+        None
+    }
+}
+
+impl<K: Ord + Clone + StepLite, V: Eq + Clone, const N: usize> From<[(RangeInclusive<K>, V); N]>
+    for RangeInclusiveMap<K, V>
+{
+    fn from(value: [(RangeInclusive<K>, V); N]) -> Self {
+        let mut map = Self::new();
+        for (range, value) in IntoIterator::into_iter(value) {
+            map.insert(range, value);
+        }
+        map
+    }
+}
+
+/// Create a [`RangeInclusiveMap`] from key-value pairs.
+///
+/// # Example
+///
+/// ```rust
+/// # use rangemap::range_inclusive_map;
+/// let map = range_inclusive_map!{
+///     0..=100 => "abc",
+///     100..=200 => "def",
+///     200..=300 => "ghi"
+/// };
+/// ```
+#[macro_export]
+macro_rules! range_inclusive_map {
+    ($($k:expr => $v:expr),* $(,)?) => {{
+        $crate::RangeInclusiveMap::from([$(($k, $v)),*])
+    }};
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
-    use alloc::{format, vec, vec::Vec};
+    use alloc as std;
+    use alloc::{format, string::String, vec, vec::Vec};
+    use proptest::prelude::*;
+    use test_strategy::proptest;
+
+    impl<K, V> Arbitrary for RangeInclusiveMap<K, V>
+    where
+        K: Ord + Clone + Debug + StepLite + Arbitrary + 'static,
+        V: Clone + PartialEq + Arbitrary + 'static,
+    {
+        type Parameters = ();
+        type Strategy = BoxedStrategy<Self>;
+
+        fn arbitrary_with(_parameters: Self::Parameters) -> Self::Strategy {
+            any::<Vec<(RangeInclusive<K>, V)>>()
+                .prop_map(|ranges| ranges.into_iter().collect::<RangeInclusiveMap<K, V>>())
+                .boxed()
+        }
+    }
+
+    #[proptest]
+    #[allow(clippy::len_zero)]
+    fn test_len(mut map: RangeInclusiveMap<u64, String>) {
+        assert_eq!(map.len(), map.iter().count());
+        assert_eq!(map.is_empty(), map.len() == 0);
+        map.clear();
+        assert_eq!(map.len(), 0);
+        assert!(map.is_empty());
+        assert_eq!(map.iter().count(), 0);
+    }
+
+    #[proptest]
+    fn test_first(set: RangeInclusiveMap<u64, String>) {
+        assert_eq!(
+            set.first_range_value(),
+            set.iter().min_by_key(|(range, _)| range.start())
+        );
+    }
+
+    #[proptest]
+    fn test_last(set: RangeInclusiveMap<u64, String>) {
+        assert_eq!(
+            set.last_range_value(),
+            set.iter().max_by_key(|(range, _)| range.end())
+        );
+    }
+
+    #[proptest]
+    fn test_iter_reversible(set: RangeInclusiveMap<u64, String>) {
+        let forward: Vec<_> = set.iter().collect();
+        let mut backward: Vec<_> = set.iter().rev().collect();
+        backward.reverse();
+        assert_eq!(forward, backward);
+    }
+
+    #[proptest]
+    fn test_into_iter_reversible(set: RangeInclusiveMap<u64, String>) {
+        let forward: Vec<_> = set.clone().into_iter().collect();
+        let mut backward: Vec<_> = set.into_iter().rev().collect();
+        backward.reverse();
+        assert_eq!(forward, backward);
+    }
+
+    #[proptest]
+    fn test_overlapping_reversible(
+        set: RangeInclusiveMap<u64, String>,
+        range: RangeInclusive<u64>,
+    ) {
+        let forward: Vec<_> = set.overlapping(&range).collect();
+        let mut backward: Vec<_> = set.overlapping(&range).rev().collect();
+        backward.reverse();
+        assert_eq!(forward, backward);
+    }
+
+    #[proptest]
+    fn test_arbitrary_map_u8(ranges: Vec<(RangeInclusive<u8>, String)>) {
+        let ranges: Vec<_> = ranges
+            .into_iter()
+            .filter(|(range, _value)| range.start() != range.end())
+            .collect();
+        let set = ranges
+            .iter()
+            .fold(RangeInclusiveMap::new(), |mut set, (range, value)| {
+                set.insert(range.clone(), value.clone());
+                set
+            });
+
+        for value in 0..u8::MAX {
+            assert_eq!(
+                set.get(&value),
+                ranges
+                    .iter()
+                    .rev()
+                    .find(|(range, _value)| range.contains(&value))
+                    .map(|(_range, value)| value)
+            );
+        }
+    }
+
+    #[proptest]
+    #[allow(deprecated)]
+    fn test_hash(left: RangeInclusiveMap<u64, u64>, right: RangeInclusiveMap<u64, u64>) {
+        use core::hash::{Hash, Hasher, SipHasher};
+
+        let hash = |set: &RangeInclusiveMap<_, _>| {
+            let mut hasher = SipHasher::new();
+            set.hash(&mut hasher);
+            hasher.finish()
+        };
+
+        if left == right {
+            assert!(
+                hash(&left) == hash(&right),
+                "if two values are equal, their hash must be equal"
+            );
+        }
+
+        // if the hashes are equal the values might not be the same (collision)
+        if hash(&left) != hash(&right) {
+            assert!(
+                left != right,
+                "if two value's hashes are not equal, they must not be equal"
+            );
+        }
+    }
+
+    #[proptest]
+    fn test_ord(left: RangeInclusiveMap<u64, u64>, right: RangeInclusiveMap<u64, u64>) {
+        assert_eq!(
+            left == right,
+            left.cmp(&right).is_eq(),
+            "ordering and equality must match"
+        );
+        assert_eq!(
+            left.cmp(&right),
+            left.partial_cmp(&right).unwrap(),
+            "ordering is total for ordered parameters"
+        );
+    }
+
+    #[test]
+    fn test_from_array() {
+        let mut map = RangeInclusiveMap::new();
+        map.insert(0..=100, "hello");
+        map.insert(200..=300, "world");
+        assert_eq!(
+            map,
+            RangeInclusiveMap::from([(0..=100, "hello"), (200..=300, "world")])
+        );
+    }
+
+    #[test]
+    fn test_macro() {
+        assert_eq!(
+            range_inclusive_map![],
+            RangeInclusiveMap::<i64, i64>::default()
+        );
+        assert_eq!(
+            range_inclusive_map!(0..=100 => "abc", 100..=200 => "def", 200..=300 => "ghi"),
+            [(0..=100, "abc"), (100..=200, "def"), (200..=300, "ghi")]
+                .iter()
+                .cloned()
+                .collect(),
+        );
+    }
 
     trait RangeInclusiveMapExt<K, V> {
         fn to_vec(&self) -> Vec<(RangeInclusive<K>, V)>;
@@ -726,7 +1137,7 @@ mod tests {
     impl<K, V> RangeInclusiveMapExt<K, V> for RangeInclusiveMap<K, V, K>
     where
         K: Ord + Clone + StepLite,
-        V: Eq + Clone,
+        V: PartialEq + Clone,
     {
         fn to_vec(&self) -> Vec<(RangeInclusive<K>, V)> {
             self.iter().map(|(kr, v)| (kr.clone(), v.clone())).collect()
@@ -1372,6 +1783,97 @@ mod tests {
         assert_eq!(gaps.next(), None);
     }
 
+    // Overlapping tests
+
+    #[test]
+    fn overlapping_ref_with_empty_map() {
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        let range_map: RangeInclusiveMap<u32, ()> = RangeInclusiveMap::new();
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆-------------◆ ◌
+        let query_range = 1..=8;
+        let mut overlapping = range_map.overlapping(&query_range);
+        // Should not yield any items.
+        assert_eq!(overlapping.next(), None);
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+    }
+
+    #[test]
+    fn overlapping_owned_with_empty_map() {
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        let range_map: RangeInclusiveMap<u32, ()> = RangeInclusiveMap::new();
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆-------------◆ ◌
+        let query_range = 1..=8;
+        let mut overlapping = range_map.overlapping(query_range);
+        // Should not yield any items.
+        assert_eq!(overlapping.next(), None);
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+    }
+
+    #[test]
+    fn overlapping_partial_edges_complete_middle() {
+        let mut range_map: RangeInclusiveMap<u32, ()> = RangeInclusiveMap::new();
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ●-● ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(0..=1, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ●-● ◌ ◌ ◌ ◌ ◌
+        range_map.insert(3..=4, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ◌ ●-● ◌ ◌
+        range_map.insert(6..=7, ());
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆---------◆ ◌ ◌ ◌
+        let query_range = 1..=6;
+
+        let mut overlapping = range_map.overlapping(&query_range);
+
+        // Should yield partially overlapped range at start.
+        assert_eq!(overlapping.next(), Some((&(0..=1), &())));
+        // Should yield completely overlapped range in middle.
+        assert_eq!(overlapping.next(), Some((&(3..=4), &())));
+        // Should yield partially overlapped range at end.
+        assert_eq!(overlapping.next(), Some((&(6..=7), &())));
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+        assert_eq!(overlapping.next(), None);
+    }
+
+    #[test]
+    fn overlapping_non_overlapping_edges_complete_middle() {
+        let mut range_map: RangeInclusiveMap<u32, ()> = RangeInclusiveMap::new();
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ●-● ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(0..=1, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ●-● ◌ ◌ ◌ ◌ ◌
+        range_map.insert(3..=4, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ◌ ●-● ◌ ◌
+        range_map.insert(6..=7, ());
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◆-----◆ ◌ ◌ ◌ ◌
+        let query_range = 2..=5;
+
+        let mut overlapping = range_map.overlapping(&query_range);
+
+        // Should only yield the completely overlapped range in middle.
+        // (Not the ranges that are touched by not covered to either side.)
+        assert_eq!(overlapping.next(), Some((&(3..=4), &())));
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+        assert_eq!(overlapping.next(), None);
+    }
+
     ///
     /// impl Debug
     ///
@@ -1393,6 +1895,14 @@ mod tests {
         assert_eq!(format!("{:?}", map), "{2..=5: (), 7..=8: (), 10..=11: ()}");
     }
 
+    // impl Default where T: ?Default
+
+    #[test]
+    fn always_default() {
+        struct NoDefault;
+        RangeInclusiveMap::<NoDefault, NoDefault>::default();
+    }
+
     // impl Serialize
 
     #[cfg(feature = "serde1")]
@@ -1420,4 +1930,18 @@ mod tests {
         let reserialized = serde_json::to_string(&range_map).expect("Failed to re-serialize");
         assert_eq!(reserialized, input);
     }
+
+    // const fn
+
+    #[cfg(feature = "const_fn")]
+    const _MAP: RangeInclusiveMap<u32, bool> = RangeInclusiveMap::new();
+    #[cfg(feature = "const_fn")]
+    const _MAP2: RangeInclusiveMap<u32, bool> = RangeInclusiveMap::new_with_step_fns();
+
+    #[cfg(feature = "quickcheck")]
+    quickcheck::quickcheck! {
+        fn prop(xs: RangeInclusiveMap<usize, usize>) -> bool {
+            xs == xs
+        }
+    }
 }
